﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/01/18 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using FVDATASLib;


namespace Affine
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;

        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

            //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;

                    //画像を表示
                    axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
                }
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }
        }

        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }

        //カメラ取り込み
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。

            axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

            try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                //画像表示
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImage1.BitPerPixel != 8)
                {
                    MessageBox.Show("グレイ画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImage1.BitPerPixel = 8;
                    axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImage1.ImageWidth = 512;
                    axFvxImage1.ImageHeight = 480;
                }

                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

        //画像ファイル保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }
        }

        //2倍拡大
        private void btnEnlarge_Click(object sender, EventArgs e)
        {
            FvPoint Pos = new FvPoint();

            //表示の始点は左上に設定
            Pos.x = 0;
            Pos.y = 0;

            //2倍に拡大
            axFvxAffine1.Magnification(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 2, 2, Pos);

            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //1/2倍縮小
        private void btnReduction_Click(object sender, EventArgs e)
        {
            FvPoint Pos = new FvPoint();

            //左上
            Pos.x = 0;
            Pos.y = 0;
            axFvxAffine1.Magnification(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 0.5, 0.5, Pos);

            // 右上
            Pos.x = axFvxImage1.ImageWidth / 2;
            Pos.y = 0;
            axFvxAffine1.Magnification(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 0.5, 0.5, Pos);

            // 左下
            Pos.x = 0;
            Pos.y = axFvxImage1.ImageHeight / 2;
            axFvxAffine1.Magnification(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 0.5, 0.5, Pos);

            // 右下
            Pos.x = axFvxImage1.ImageWidth / 2;
            Pos.y = axFvxImage1.ImageHeight / 2;
            axFvxAffine1.Magnification(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), 0.5, 0.5, Pos);

            axFvxImage2.CopyTo(axFvxImage1.GetOcx());
            axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
        }

        //領域を指定して拡大
        private void btnAreaEnlarge_Click(object sender, EventArgs e)
        {
            double dblScaleX = 0;
            double dblScaleY = 0;
            double dblTemp = 0;
            int intOk = 0;

            FvRectangle Rect = new FvRectangle();
            FvPoint Pos = new FvPoint();
            FvMovement Move = new FvMovement();

            //範囲選択の初期値を適当に設定
            Rect.xs = 100;
            Rect.ys = 100;
            Rect.xe = 200;
            Rect.ye = 200;
            axFvxROI1.SetRectangle(Rect);

            //FvxROIを使って処理範囲選択ダイアログを出します
            axFvxROI1.UsePoint = false;
            axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out intOk);

            if (intOk == 1 && axFvxROI1.CheckRectangle == 1)
            {
                axFvxROI1.GetRectangle(Rect);

                // 始点と終点が逆になっていないかチェックします｡
                // FvxROIは矩形データで、左上・右下の座標位置が逆になっている場合(xs > xe, ys > ye)でも、
                // 入れ替えずそのまま結果を返しますので、注意してください
                if (Rect.xe < Rect.xs)
                {
                    dblTemp = Rect.xs;
                    Rect.xs = Rect.xe;
                    Rect.xe = dblTemp;
                }
                if (Rect.ye < Rect.ys)
                {
                    dblTemp = Rect.ys;
                    Rect.ys = Rect.ye;
                    Rect.ye = dblTemp;
                }

                //拡大の倍率計算
                dblScaleX = axFvxImage1.ImageWidth / (Rect.xe - Rect.xs);
                dblScaleY = axFvxImage1.ImageHeight / (Rect.ye - Rect.ys);

                //拡大領域の左上が座標(0,0)にくるように平行移動
                Move.x = Rect.xs;
                Move.y = Rect.ys;
                axFvxAffine1.Shift(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), Move);

                //表示の始点は左上
                Pos.x = 0;
                Pos.y = 0;
                axFvxAffine1.Magnification(axFvxImage2.GetOcx(), axFvxImage1.GetOcx(), dblScaleX, dblScaleY, Pos);

                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

        //平行移動
        private void btnShift_Click(object sender, EventArgs e)
        {
            frmShift dlg = new frmShift();
            FvMovement Move = new FvMovement();

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                //移動量設定
                Move.x = dlg.MoveX;
                Move.y = dlg.MoveY;

                axFvxAffine1.Shift(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), Move);

                axFvxImage2.CopyTo(axFvxImage1.GetOcx());
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

        //回転
        private void btnRotate_Click(object sender, EventArgs e)
        {
            frmRotate dlg = new frmRotate();
            FvPoint Pos = new FvPoint();

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                //画像の中心を回転中心とする
                Pos.x = axFvxImage1.ImageWidth / 2;
                Pos.y = axFvxImage1.ImageHeight / 2;

                axFvxAffine1.Rotation(axFvxImage1.GetOcx(), axFvxImage2.GetOcx(), dlg.Angle, Pos);

                axFvxImage2.CopyTo(axFvxImage1.GetOcx());
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

    }
    
}